@@ -1,4 +1,10 @@
 """This module contains constants used by other modules."""
+MAJOR_VERSION = 2
+MINOR_VERSION = 2
+PATCH_VERSION = 1
+__short_version__ = '{}.{}'.format(MAJOR_VERSION, MINOR_VERSION)
+__version__ = '{}.{}'.format(__short_version__, PATCH_VERSION)
+REQUIRED_PYTHON_VER = (3, 5)
 
 # ### Misc ###
 STANDALONE_AID = 1  # Standalone accessory ID (i.e. not bridged)
